@@ -1,5 +1,5 @@
-require 'set'
-require 'shellwords'
+require "set"
+require "shellwords"
 
 module Utilrb
     # Access to information from pkg-config(1)
@@ -320,17 +320,28 @@ module Utilrb
             end
         end
 
+        def builtin_variables(path)
+            {
+                "pcfiledir" => File.dirname(path),
+                "pc_sysrootdir" => sysrootdir
+            }
+        end
+
         def load_variables(path, preset_variables = Hash.new)
             raw_variables, raw_fields = parse(path)
             raw_variables = preset_variables.merge(raw_variables)
-            expand_variables(raw_variables)
+            expand_variables(
+                raw_variables.merge(builtin_variables(path))
+            )
         end
         
         def load_minimal(path, preset_variables = Hash.new)
             raw_variables, raw_fields = parse(path)
             raw_variables = preset_variables.merge(raw_variables)
 
-            @variables = expand_variables(raw_variables)
+            @variables = expand_variables(
+                raw_variables.merge(builtin_variables(path))
+            )
             if raw_fields['Version']
                 @raw_version = expand_field('Version', raw_fields['Version'])
             else
@@ -362,22 +373,31 @@ module Utilrb
                 fields['Conflicts'] || '', allow_loops: true, memo: memo)
 
             # And finally resolve the compilation flags
-            @cflags = fields['Cflags'] || []
+            cflags = fields['Cflags'] || []
+            cflags.uniq!
+            cflags -= self.class.default_search_path_I
+            cflags = apply_sysrootdir(cflags, "-I")
             @requires.each do |pkg|
-                @cflags.concat(pkg.raw_cflags)
+                cflags.concat(pkg.raw_cflags)
             end
             @requires_private.each do |pkg|
-                @cflags.concat(pkg.raw_cflags)
+                cflags.concat(pkg.raw_cflags)
             end
-            @cflags.uniq!
-            @cflags.delete('-I/usr/include')
-            @ldflags = Hash.new
-            @ldflags[false] = fields['Libs'] || []
-            @ldflags[false].delete('-L/usr/lib')
-            @ldflags[false].uniq!
-            @ldflags[true] = @ldflags[false] + (fields['Libs.private'] || [])
-            @ldflags[true].delete('-L/usr/lib')
-            @ldflags[true].uniq!
+            @cflags = cflags
+
+            ldflags_public = fields['Libs'] || []
+            ldflags_public.uniq!
+            ldflags_private = ldflags_public + (fields['Libs.private'] || [])
+            ldflags_private.uniq!
+
+            ldflags_public -= self.class.default_search_path_L
+            ldflags_public = apply_sysrootdir(ldflags_public, "-L")
+            ldflags_private -= self.class.default_search_path_L
+            ldflags_private = apply_sysrootdir(ldflags_private, "-L")
+            @ldflags = {
+                false => ldflags_public,
+                true => ldflags_private
+            }
 
             @ldflags_with_requires = {
                 true => @ldflags[true].dup,
@@ -424,6 +444,7 @@ module Utilrb
             if result.any?(&:empty?)
                 raise Invalid.new(name), "empty include directory (-I without argument) found in pkg-config package #{name}"
             end
+
             result
         end
 
@@ -434,12 +455,34 @@ module Utilrb
             if result.any?(&:empty?)
                 raise Invalid.new(name), "empty link directory (-L without argument) found in pkg-config package #{name}"
             end
+
             result
         end
 
+        # A "new root" that should be prepended to -L and -I flags
+        def sysrootdir
+            ENV["PKG_CONFIG_SYSROOT_DIR"] || "/"
+        end
+
+        # @api private
+        #
+        # Apply {#sysrootdir} to all the given paths flags (-I or -L)
+        def apply_sysrootdir(entries, flag_name)
+            sysrootdir = self.sysrootdir
+            return entries if sysrootdir == "/"
+
+            entries.map do |v|
+                if v.start_with?(flag_name)
+                    "#{flag_name}#{sysrootdir}#{v[2..-1]}"
+                else
+                    v
+                end
+            end
+        end
+
 	ACTIONS = %w{cflags cflags-only-I cflags-only-other 
 		    libs libs-only-L libs-only-l libs-only-other}
-    ACTIONS.each { |action| define_pkgconfig_action(action) }
+        ACTIONS.each { |action| define_pkgconfig_action(action) }
     
         def conflicts
             @conflicts
@@ -585,13 +628,52 @@ module Utilrb
         def self.default_search_path
             if !@default_search_path
                 output = `LANG=C PKG_CONFIG_PATH= pkg-config --debug 2>&1`.split("\n")
-                @default_search_path = output.grep(FOUND_PATH_RX).
-                    map { |l| l.gsub(FOUND_PATH_RX, '\1\2') }
+                @default_search_path =
+                    output.grep(FOUND_PATH_RX)
+                    .map { |l| l.gsub(FOUND_PATH_RX, '\1\2') }
             end
             return @default_search_path
         end
         @default_search_path = nil
 
+        def self.arch_dir
+            return if @arch_dir == false
+
+            unless @arch_dir
+                suffix_with_arch =
+                    default_search_suffixes
+                    .find { |p| %r{^lib/[^/]+/pkgconfig} =~ p }
+
+                @arch_dir =
+                    if suffix_with_arch
+                        suffix_with_arch.split("/")[1]
+                    else
+                        false
+                    end
+            end
+
+            @arch_dir
+        end
+
+        def self.default_search_path_L
+            unless @default_search_path_L
+                arch_dir = self.arch_dir
+                @default_search_path_L =
+                    ["-L/usr/lib", "-L/lib"]
+                    .flat_map { |p| [p, "#{p}/#{arch_dir}"] }
+            end
+
+            @default_search_path_L
+        end
+
+        def self.default_search_path_I
+            unless @default_search_path_I
+                @default_search_path_I = ["-I/usr/include"]
+            end
+
+            @default_search_path_I
+        end
+
         # Returns the system-wide standard suffixes that should be appended to
         # new prefixes to find pkg-config files
         def self.default_search_suffixes
